@@ -27,29 +27,20 @@ using Rcpp::NumericVector;
 *
 */
 
-inline double pmf_bbinom(double k, double n, double alpha,
-                         double beta, bool& throw_warning) {
-  if (ISNAN(k) || ISNAN(n) || ISNAN(alpha) || ISNAN(beta))
-    return k+n+alpha+beta;
-  if (alpha < 0.0 || beta < 0.0 || n < 0.0 || !isInteger(n, false)) {
-    throw_warning = true;
-    return NAN;
-  }
-  if (!isInteger(k) || k < 0.0 || k > n)
-    return 0.0;
-  return R::choose(n, k) * R::beta(k+alpha, n-k+beta) / R::beta(alpha, beta);
-}
 
 inline double logpmf_bbinom(double k, double n, double alpha,
                             double beta, bool& throw_warning) {
+#ifdef IEEE_754
   if (ISNAN(k) || ISNAN(n) || ISNAN(alpha) || ISNAN(beta))
     return k+n+alpha+beta;
+#endif
   if (alpha < 0.0 || beta < 0.0 || n < 0.0 || !isInteger(n, false)) {
     throw_warning = true;
     return NAN;
   }
   if (!isInteger(k) || k < 0.0 || k > n)
     return R_NegInf;
+  // R::choose(n, k) * R::beta(k+alpha, n-k+beta) / R::beta(alpha, beta);
   return R::lchoose(n, k) + R::lbeta(k+alpha, n-k+beta) - R::lbeta(alpha, beta);
 }
 
@@ -187,10 +178,15 @@ NumericVector cpp_pbbinom(
     if (i % 100 == 0)
       Rcpp::checkUserInterrupt();
     
+#ifdef IEEE_754
     if (ISNAN(GETV(x, i)) || ISNAN(GETV(size, i)) ||
         ISNAN(GETV(alpha, i)) || ISNAN(GETV(beta, i))) {
       p[i] = GETV(x, i) + GETV(size, i) + GETV(alpha, i) + GETV(beta, i);
-    } else if (GETV(alpha, i) <= 0.0 || GETV(beta, i) <= 0.0 ||
+      continue;
+    }
+#endif
+    
+    if (GETV(alpha, i) <= 0.0 || GETV(beta, i) <= 0.0 ||
                GETV(size, i) < 0.0 || !isInteger(GETV(size, i), false)) {
       throw_warning = true;
       p[i] = NAN;
