@@ -27,36 +27,28 @@ using Rcpp::NumericVector;
  *
  */
 
-inline double pdf_pareto(double x, double a, double b,
-                         bool& throw_warning) {
-  if (ISNAN(x) || ISNAN(a) || ISNAN(b))
-    return x+a+b;
-  if (a <= 0.0 || b <= 0.0) {
-    throw_warning = true;
-    return NAN;
-  }
-  if (x < b)
-    return 0.0;
-  return a * pow(b, a) / pow(x, a+1.0);
-}
-
 inline double logpdf_pareto(double x, double a, double b,
                             bool& throw_warning) {
+#ifdef IEEE_754
   if (ISNAN(x) || ISNAN(a) || ISNAN(b))
     return x+a+b;
+#endif
   if (a <= 0.0 || b <= 0.0) {
     throw_warning = true;
     return NAN;
   }
   if (x < b)
     return R_NegInf;
+  // a * pow(b, a) / pow(x, a+1.0);
   return log(a) + log(b)*a - log(x)*(a+1.0);
 }
 
 inline double cdf_pareto(double x, double a, double b,
                          bool& throw_warning) {
+#ifdef IEEE_754
   if (ISNAN(x) || ISNAN(a) || ISNAN(b))
     return x+a+b;
+#endif
   if (a <= 0.0 || b <= 0.0) {
     throw_warning = true;
     return NAN;
@@ -68,8 +60,10 @@ inline double cdf_pareto(double x, double a, double b,
 
 inline double invcdf_pareto(double p, double a, double b,
                             bool& throw_warning) {
+#ifdef IEEE_754
   if (ISNAN(p) || ISNAN(a) || ISNAN(b))
     return p+a+b;
+#endif
   if (a <= 0.0 || b <= 0.0 || !VALID_PROB(p)) {
     throw_warning = true;
     return NAN;
