@@ -22,7 +22,7 @@ using Rcpp::NumericVector;
 *  b > 0
 *  eta > 0
 *
-*  f(x)    = b*exp(-b*x) * exp(-eta*exp(-b*x)) * exp(1 + eta*(1 - exp(-b*x)))
+*  f(x)    = b*exp(-b*x) * exp(-eta*exp(-b*x)) * (1 + eta*(1 - exp(-b*x)))
 *  F(x)    = (1-exp(-b*x)) * exp(-eta*exp(-b*x))
 *
 * References:
@@ -44,24 +44,29 @@ using Rcpp::NumericVector;
 */
 
 
-inline double pdf_sgomp(double x, double b, double eta,
+inline double logpdf_sgomp(double x, double b, double eta,
                            bool& throw_warning) {
+#ifdef IEEE_754
   if (ISNAN(x) || ISNAN(b) || ISNAN(eta))
     return x+b+eta;
+#endif
   if (b <= 0.0 || eta <= 0.0) {
     throw_warning = true;
     return NAN;
   }
   if (x < 0.0 || !R_FINITE(x))
-    return 0.0;
+    return R_NegInf;
   double ebx = exp(-b*x);
-  return b*ebx * exp(-eta*ebx) * (1+eta*(1-ebx));
+  // b*ebx * exp(-eta*ebx) * (1+eta*(1-ebx));
+  return log(b) + log(ebx) - eta*ebx + log1p(eta*(1-ebx));
 }
 
 inline double cdf_sgomp(double x, double b, double eta,
                            bool& throw_warning) {
+#ifdef IEEE_754
   if (ISNAN(x) || ISNAN(b) || ISNAN(eta))
     return x+b+eta;
+#endif
   if (b <= 0.0 || eta <= 0.0) {
     throw_warning = true;
     return NAN;
@@ -71,7 +76,8 @@ inline double cdf_sgomp(double x, double b, double eta,
   if (x == R_PosInf)
     return 1.0;
   double ebx = exp(-b*x);
-  return (1-ebx) * exp(-eta*ebx);
+  // (1-ebx) * exp(-eta*ebx)
+  return exp(log1p(-ebx) - eta*ebx);
 }
 
 inline double rng_sgomp(double b, double eta, bool& throw_warning) {
@@ -79,10 +85,11 @@ inline double rng_sgomp(double b, double eta, bool& throw_warning) {
     throw_warning = true;
     return NA_REAL;
   }
-  double u, rg, re;
-  u = rng_unif();
-  rg = -log( -log(u)/eta ) / b;
-  re = exp_rand() / b;
+  double u, v, rg, re;
+  u = R::exp_rand(); // -log(rng_unif())
+  v = R::exp_rand(); // -log(rng_unif())
+  rg = -log(u/eta) / b;
+  re = v / b;
   return (rg>re) ? rg : re;
 }
 
@@ -109,11 +116,11 @@ NumericVector cpp_dsgomp(
   bool throw_warning = false;
   
   for (int i = 0; i < Nmax; i++)
-    p[i] = pdf_sgomp(GETV(x, i), GETV(b, i),
-                     GETV(eta, i), throw_warning);
+    p[i] = logpdf_sgomp(GETV(x, i), GETV(b, i),
+                        GETV(eta, i), throw_warning);
   
-  if (log_prob)
-    p = Rcpp::log(p);
+  if (!log_prob)
+    p = Rcpp::exp(p);
   
   if (throw_warning)
     Rcpp::warning("NaNs produced");
