@@ -27,8 +27,10 @@ using Rcpp::NumericVector;
 
 inline double pdf_wald(double x, double mu, double lambda,
                        bool& throw_warning) {
+#ifdef IEEE_754
   if (ISNAN(x) || ISNAN(mu) || ISNAN(lambda))
     return x+mu+lambda;
+#endif
   if (mu <= 0.0 || lambda <= 0.0) {
     throw_warning = true;
     return NAN;
@@ -36,13 +38,15 @@ inline double pdf_wald(double x, double mu, double lambda,
   if (x <= 0.0 || !R_FINITE(x))
     return 0.0;
   return sqrt(lambda/(2.0*PI*(x*x*x))) *
-         exp((-lambda*(x-mu)*(x-mu))/(2.0*(mu*mu)*x));
+         exp( (-lambda*(x-mu)*(x-mu))/(2.0*(mu*mu)*x) );
 }
 
 inline double cdf_wald(double x, double mu, double lambda,
                        bool& throw_warning) {
+#ifdef IEEE_754
   if (ISNAN(x) || ISNAN(mu) || ISNAN(lambda))
     return x+mu+lambda;
+#endif
   if (mu <= 0.0 || lambda <= 0.0) {
     throw_warning = true;
     return NAN;
